﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;

namespace TestProject;

/// <summary>
/// 이미지 회전 목마 컨트롤
/// </summary>
public class ImageCarousel : Control
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 이전 화살표 버튼
    /// </summary>
    ArrowButton previousArrowButton;

    /// <summary>
    /// 라운드 이미지
    /// </summary>
    private RoundImage roundImage;

    /// <summary>
    /// 설명 텍스트 블럭
    /// </summary>
    private TextBlock descriptionTextBlock;

    /// <summary>
    /// 다음 화살표 버튼
    /// </summary>
    ArrowButton nextArrowButton;

    /// <summary>
    /// 이미지 항목 리스트
    /// </summary>
    private List<ImageItem> imageItemList;

    /// <summary>
    /// 현재 이미지 항목 인덱스
    /// </summary>
    private int currentImageItemIndex;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Static

    #region 생성자 - ImageCarousel()

    /// <summary>
    /// 생성자
    /// </summary>
    static ImageCarousel()
    {
        DefaultStyleKeyProperty.OverrideMetadata(typeof(ImageCarousel), new FrameworkPropertyMetadata(typeof(ImageCarousel)));
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - ImageCarousel()

    /// <summary>
    /// 생성자
    /// </summary>
    public ImageCarousel()
    {
        this.imageItemList = new List<ImageItem>();

        this.currentImageItemIndex = 0;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public
    //////////////////////////////////////////////////////////////////////////////// Function

    #region 템플리트 적용시 처리하기 - OnApplyTemplate()

    /// <summary>
    /// 템플리트 적용시 처리하기
    /// </summary>
    public override void OnApplyTemplate()
    {
        base.OnApplyTemplate();

        this.previousArrowButton  = GetTemplateChild("PART_PreviousArrowButton" ) as ArrowButton;
        this.roundImage           = GetTemplateChild("PART_RoundImage"          ) as RoundImage;
        this.descriptionTextBlock = GetTemplateChild("PART_DescriptionTextBlock") as TextBlock;
        this.nextArrowButton      = GetTemplateChild("PART_NextArrowButton"     ) as ArrowButton;

        this.previousArrowButton.Click += previousArrowButton_Click;
        this.nextArrowButton.Click     += nextArrowButton_Click;

        DisplayCurrentImage();
    }

    #endregion
    #region 이미지 추가하기 - AddImage(imageURI, description)

    /// <summary>
    /// 이미지 추가하기
    /// </summary>
    /// <param name="imageURI">이미지 URI</param>
    /// <param name="description">설명</param>
    public void AddImage(Uri imageURI, string description)
    {
        this.imageItemList.Add(new ImageItem { ImageURI = imageURI, Description = description });

        if(this.imageItemList.Count == 1)
        {
            DisplayCurrentImage();
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region 이전 화살표 버튼 클릭시 처리하기 - previousArrowButton_Click(sender, e)

    /// <summary>
    /// 이전 화살표 버튼 클릭시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void previousArrowButton_Click(object sender, RoutedEventArgs e)
    {
        if(this.imageItemList.Count > 0)
        {
            this.currentImageItemIndex = (this.currentImageItemIndex - 1 + this.imageItemList.Count) % this.imageItemList.Count;

            DisplayCurrentImage();
        }
    }

    #endregion
    #region 다음 화살표 버튼 클릭시 처리하기 - nextArrowButton_Click(sender, e)

    /// <summary>
    /// 다음 화살표 버튼 클릭시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void nextArrowButton_Click(object sender, RoutedEventArgs e)
    {
        if(this.imageItemList.Count > 0)
        {
            this.currentImageItemIndex = (this.currentImageItemIndex + 1) % this.imageItemList.Count;

            DisplayCurrentImage();
        }
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region 현재 이미지 표시하기 - DisplayCurrentImage()

    /// <summary>
    /// 현재 이미지 표시하기
    /// </summary>
    private void DisplayCurrentImage()
    {
        if(this.imageItemList.Count > 0 && this.roundImage != null && this.descriptionTextBlock != null)
        {
            ImageItem currentImageItem = this.imageItemList[this.currentImageItemIndex];

            this.roundImage.ImageSource = new BitmapImage(currentImageItem.ImageURI);

            this.descriptionTextBlock.Text = currentImageItem.Description;
        }
    }

    #endregion
}