﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;

namespace TestProject;

/// <summary>
/// 라운드 이미지 컨트롤
/// </summary>
public class RoundImage : Control
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Dependency Property
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 코너 반경 속성 - CornerRadiusProperty

    /// <summary>
    /// 코너 반경 속성
    /// </summary>
    public static readonly DependencyProperty CornerRadiusProperty = DependencyProperty.Register
    (
        "CornerRadius",
        typeof(CornerRadius),
        typeof(RoundImage),
        new PropertyMetadata(new CornerRadius(10))
    );

    #endregion
    #region 스트레치 속성 - StretchProperty

    /// <summary>
    /// 스트레치 속성
    /// </summary>
    public static readonly DependencyProperty StretchProperty = DependencyProperty.Register
    (
        "Stretch",
        typeof(Stretch),
        typeof(RoundImage),
        new PropertyMetadata(Stretch.UniformToFill, StretchPropertyChangedCallback)
    );

    #endregion
    #region 이미지 소스 속성 - ImageSourceProperty

    /// <summary>
    /// 이미지 소스 속성
    /// </summary>
    public static readonly DependencyProperty ImageSourceProperty = DependencyProperty.Register
    (
        "ImageSource",
        typeof(ImageSource),
        typeof(RoundImage),
        new PropertyMetadata(null, ImageSourcePropertyChangedCallback)
    );

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 루트 테두리
    /// </summary>
    private Border rootBorder = null;

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Property
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 코너 반경 - CornerRadius

    /// <summary>
    /// 코너 반경
    /// </summary>
    public CornerRadius CornerRadius
    {
        get => (CornerRadius)GetValue(CornerRadiusProperty);
        set => SetValue(CornerRadiusProperty, value);
    }

    #endregion
    #region 스트레치 - Stretch

    /// <summary>
    /// 스트레치
    /// </summary>
    public Stretch Stretch
    {
        get => (Stretch)GetValue(StretchProperty);
        set => SetValue(StretchProperty, value);
    }

    #endregion
    #region 이미지 소스 - ImageSource

    /// <summary>
    /// 이미지 소스
    /// </summary>
    public ImageSource ImageSource
    {
        get => (ImageSource)GetValue(ImageSourceProperty);
        set => SetValue(ImageSourceProperty, value);
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Static

    #region 생성자 - RoundImage()

    /// <summary>
    /// 생성자
    /// </summary>
    static RoundImage()
    {
        DefaultStyleKeyProperty.OverrideMetadata(typeof(RoundImage), new FrameworkPropertyMetadata(typeof(RoundImage)));
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - RoundImage()

    /// <summary>
    /// 생성자
    /// </summary>
    public RoundImage()
    {
        Loaded += Control_Loaded;
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static

    #region 스트레치 속성 변경시 콜백 처리하기 - StretchPropertyChangedCallback(d, e)

    /// <summary>
    /// 스트레치 속성 변경시 콜백 처리하기
    /// </summary>
    /// <param name="d">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private static void StretchPropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
        if(d is RoundImage roundImage)
        {
            roundImage.UpdateBackground();
        }
    }

    #endregion
    #region 이미지 소스 속성 변경시 콜백 처리하기 - ImageSourcePropertyChangedCallback(d, e)

    /// <summary>
    /// 이미지 소스 속성 변경시 콜백 처리하기
    /// </summary>
    /// <param name="d">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private static void ImageSourcePropertyChangedCallback(DependencyObject d, DependencyPropertyChangedEventArgs e)
    {
        if(d is RoundImage roundImage)
        {
            roundImage.UpdateBackground();
        }
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Instance
    //////////////////////////////////////////////////////////////////////////////// Public
    ////////////////////////////////////////////////////////////////////// Function

    #region 템플리트 적용시 처리하기 - OnApplyTemplate()

    /// <summary>
    /// 템플리트 적용시 처리하기
    /// </summary>
    public override void OnApplyTemplate()
    {
        base.OnApplyTemplate();

        this.rootBorder = GetTemplateChild("PART_RootBorder") as Border;
    }

    #endregion

    ////////////////////////////////////////////////////////////////////////////////////////// Private
    //////////////////////////////////////////////////////////////////////////////// Event

    #region 컨트롤 로드시 처리하기 - Control_Loaded(sender, e)

    /// <summary>
    /// 컨트롤 로드시 처리하기
    /// </summary>
    /// <param name="sender">이벤트 발생자</param>
    /// <param name="e">이벤트 인자</param>
    private void Control_Loaded(object sender, RoutedEventArgs e)
    {
        UpdateBackground();
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////// Function

    #region 배경 업데이트하기 - UpdateBackground()

    /// <summary>
    /// 배경 업데이트하기
    /// </summary>
    private void UpdateBackground()
    {
        if(this.rootBorder == null)
        {
            return;
        }

        if(ImageSource != null)
        {
            this.rootBorder.Background = new ImageBrush
            {
                ImageSource = this.ImageSource,
                Stretch = this.Stretch
            };
        }
        else
        {
            this.rootBorder.Background = null;
        }
    }

    #endregion
}